local mod = MorgDKP2:NewModule("PointsDB")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2")
local BCR = LibStub("LibBabble-Class-3.0"):GetReverseLookupTable()

local mdkp = MorgDKP2
mod.modName = "PointsDB"

--locals
local LL = LibStub("AceLocale-3.0"):NewLocale("PointsDB", "enUS", true)

if LL then
	LL["Clear player DB"] = true
	LL["Clears all players currently in the database. Does not affect items."] = true
	LL["Clear raid DB"] = true
	LL["Clears the current tracked raids."] = true
	LL["Database Functions"] = true
	LL["Import database from MorgDKP?"] = true
	LL["Imported MorgDKP DB with %s players and %s items."] = true
	LL["Import default item database?"] = true
	LL["Database transfer complete. %s items were transferred and %s members updated."] = true
	LL["Please delete the ItemData.lua file from the MorgDKP directory now."] = true
	LL["Updating DKP Points.."] = true
	LL["Added alias: %s of %s with %s DKP."] = true
	LL["Added %s players from %s"] = true
	LL["Player database cleared!"] = true
	LL["Ignored items"] = true
	LL["List of currently ignored items.  Click to remove item from ignore list."] = true
	LL["Zero player DB"] = true
	LL["Resets all members DKP to zero."] = true
	LL["Player DKP reset to zero!"] = true
	LL["Cull item DB"] = true
	LL["Clears all items in the database below the selected item level."] = true
	LL["Cull item level"] = true
	LL["Add item"] = true
	LL["Add new item to track.  Drag the item to this input box."] = true
	LL["Addeded %s to item database."] = true
	LL["Cull member DB"] = true
	LL["Clears all members in the database inactive longer than the selected number of weeks."] = true
	LL["Cull member level (weeks)"] = true
	LL["Removed %s items from the database."] = true
	LL["Removed %s members from the database."] = true
end

local LL = LibStub("AceLocale-3.0"):GetLocale("PointsDB")

local db
local band = bit.band
local bor = bit.bor
local bxor = bit.bxor
local fmt = string.format
local upper = string.upper

local defaults = { 
	profile = {
		culllvl = 200, 
		memculllvl = 4
   	},
}

local options = {
		type = "group",
		handler = mod,
		name = LL["Database Functions"],
		desc = LL["Database Functions"],
		order = 200,
		args = {
			clearraiddb = {
				type = "execute",
				name = LL["Clear raid DB"],
				desc = LL["Clears the current tracked raids."],
				func = function() mod:ClearRaidDB() end,
				disabled = function() return not mod:IsEnabled() end,
				order = 1
			},
			clear = {
				type = "execute",
				name = LL["Clear player DB"],
				desc = LL["Clears all players currently in the database. Does not affect items."],
				func = function() mod:ClearDatabase() end,
				disabled = function() return not mod:IsEnabled() end,
				order = 3
			},
			zeroplayerdb = {
				type = "execute",
				name = LL["Zero player DB"],
				desc = LL["Resets all members DKP to zero."],
				func = function() mod:ZeroPLayerDB() end,
				disabled = function() return not mod:IsEnabled() end,
				order = 4
			},
			head = {
				type = "header",
				name = "",
				order = 6
			},
			additem = {
				type = "input",
				name = LL["Add item"],
				desc = LL["Add new item to track.  Drag the item to this input box."],
				set = 	function(info, v) mod:AddNewItem(v) end,
				order = 8
			},
			desc = {
				type = "description",
				name = "",
				order = 9
			},
			cullllvl = {
				type = "range",
				name = LL["Cull item level"],
				get = function() return mod.db.profile.culllvl end,
				set = 	function(info, v) mod.db.profile.culllvl = v end,
				min = 2, 
				max = 300, 
				step = 1,
				hidden = function() return not mod:IsEnabled() end,
				order = 10
			},
			cullitemdb = {
				type = "execute",
				name = LL["Cull item DB"],
				desc = LL["Clears all items in the database below the selected item level."],
				func = function() mod:CullItemDatabase() end,
				disabled = function() return not mod:IsEnabled() end,
				order = 15
			},
			desc2 = {
				type = "description",
				name = "",
				order = 16
			},
			cullmemllvl = {
				type = "range",
				name = LL["Cull member level (weeks)"],
				get = function() return mod.db.profile.memculllvl end,
				set = 	function(info, v) mod.db.profile.memculllvl = v end,
				min = 1, 
				max = 16, 
				step = 1,
				hidden = function() return not mod:IsEnabled() end,
				order = 17
			},
			cullmemberdb = {
				type = "execute",
				name = LL["Cull member DB"],
				desc = LL["Clears all members in the database inactive longer than the selected number of weeks."],
				func = function() mod:CullMemberDatabase() end,
				disabled = function() return not mod:IsEnabled() end,
				order = 18
			},
			head2 = {
				type = "header",
				name = "",
				order = 20
			},
			ignore = {
				type = "select",
				name = LL["Ignored items"],
				desc = LL["List of currently ignored items.  Click to remove item from ignore list."],
				get = false,
				set = function(info, v) 
						table.remove(mdkp.db.profile.ignore, v) 
					end,
				values = function() return mdkp.db.profile.ignore end,
				disabled = function() return not mod:IsEnabled() or not mdkp.db.profile.ignore[1] end,
				order = 50
			},
        	},
}

StaticPopupDialogs["ImportMDKP"] = {
	text = TEXT(LL["Import database from MorgDKP?"]),
	button1 = TEXT(YES),
	button2 = TEXT(NO),
	OnAccept = function()
		mod:ImportMorgDKPDB()
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
}

StaticPopupDialogs["ImportMorgItems"] = {
	text = TEXT(LL["Import default item database?"]),
	button1 = TEXT(YES),
	button2 = TEXT(NO),
	OnAccept = function()
		mod:ImportMorgItems()
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
}

function mod:GetOptions()
	return options
end

function mod:IsDKP()
	return nil
end

function mod:OnInitialize()
	self.db = mdkp.db:RegisterNamespace("PointsDB", defaults)
	
end

function mod:OnEnable()
	db = mdkp.db.profile
	db.moduleON["PointsDB"] = true
	if MorgDKP and not db.dbupdated then	--import morgdkp db?
		StaticPopup_Show ("ImportMDKP")
		self.importingDB = true
	end
	if MorgDKP2_Items then StaticPopup_Show ("ImportMorgItems") end	--import item db?
	self:ImportDKPInfoFile()
end

function mod:OnDisable()
	db.moduleON["PointsDB"] = nil
end

function mod:ClearDatabase()
	self:debug("Clear DB")
	db.info = { }
	self:out(LL["Player database cleared!"])
end

function mod:ZeroPLayerDB()
	local POOL = self:GetPool()
	for name, data in pairs(db.info) do
		data[POOL] = {points = 0, earned = 0, spent = 0}
		data.lastupdate = self:GetServerTime()
	end
	self:out(LL["Player DKP reset to zero!"])
end

function mod:ClearRaidDB()
	local tracker = self:GetModuleRef("Tracker")
	if tracker then tracker:ClearRaidDB(true) end
end

function mod:AddNewItem(itemlink)
	if not itemlink or itemlink == "" then return end
	local _,_, item = string.find(itemlink, "item:(%d+):")
	item = tonumber(item)
	self:IteminDB(item)
	self:out(fmt(LL["Addeded %s to item database."], itemlink))
end

function mod:ImportMorgDKPDB()
	self:debug("Import MorgDKP")
	local POOL = self:GetPool()
	local update = self:GetServerTime()
	local playercount = 0
	local itemcount = 0
	if not POOL or POOL == L["NONE"] then	--no DKP info imported yet
		db.raid = MorgDKP.db.account.defaults.raid or L["NONE"]
		POOL = db.raid
		db.raidlist = MorgDKP.db.account.defaults.raidlist or L["NONE"]
		db.disenchanter = MorgDKP.db.account.defaults.disenchanter or L["NONE"]
		db.eqDKP = MorgDKP.db.account.defaults.eqDKP
	end
	for index, v in pairs(MorgDKP.db.account.info) do
		local temp = { }
		temp.raidloot = 0
		temp.lastupdate = update
		temp.aliases = v.aliases
		temp.items = {}
		local buffer = {}
		if v.items then
			for _, ID in pairs(v.items) do
				if not buffer[ID] then
					tinsert(temp.items, ID)
					buffer[ID] = true
				end
			end
		end
		if v.class then 
			local englishclass = upper(BCR[v.class]) 
			if englishclass then temp.class = englishclass end
		end
		if v[POOL] then 
			temp[POOL] = {points = v[POOL].points, earned = v[POOL].earned, spent = v[POOL].spent}
		else temp[POOL] = {points = 0, earned = 0, spent = 0} end
		db.info[index] = temp
		playercount = playercount + 1
	end
	for id, data in pairs(MorgDKP.db.account.items) do
		if not db.items[id] then
			local temp = {}
			temp.classes = 0
			temp.altclasses = 0
			--if not data.pool then db.items[id].pool = POOL end	shouldnt need default pool
			temp.points = data.points
			for class, val in pairs(data.classes) do
				if val == true then
					bitop = mdkp.Bits[string.upper(class)]
					temp.classes = bor(bitop, temp.classes)
				end
			end
			for class, val in pairs(data.altclasses) do
				if val == true then
					bitop = mdkp.Bits[string.upper(class)]
					temp.altclasses = bor(bitop, temp.altclasses)
				end
			end
			db.items[id] = temp
			itemcount = itemcount + 1
		end
	end
	db.dbupdated = 0
	self:out(fmt(LL["Imported MorgDKP DB with %s players and %s items."], playercount, itemcount))
	mdkp:ClearRaidDB()
	self.importingDB = nil
	self:ImportDKPInfoFile()
end

function mod:ImportMorgItems()
	itemcount = self:ImportItemDB(MorgDKP2_Items)
	self:out(fmt(LL["Database transfer complete. %s items were transferred and %s members updated."], itemcount, 0))
	self:out(LL["Please delete the ItemData.lua file from the MorgDKP directory now."])
	MorgDKP2_Items = nil
end

function mod:ImportItemDB(items)
	local itemcount = 0
	if not db.items then db.items = { } end
	for id, data in pairs(items) do
		if not db.items[id] then
			db.items[id] = data
			itemcount = itemcount + 1
		end
	end
	return itemcount
end

function mod:ImportDKPInfoFile()
	local db = mdkp.db.profile
	if self.importingDB then return end
	local temp = {}
	local count = 1
	if not MorgDKP2_DKP1 then self:DefaultDKP() end
	if not MorgDKP2_DKP1 then return end
	for i = 1,5 do
		local dkp = getglobal("MorgDKP2_DKP" .. i)
		if  dkp then temp[count] =  dkp
			count = count + 1
		end
	end
	local skipupdate = 1
	local timestamp
	if not db.Timestamp or self:TimeStampDiff(temp[1].Timestamp, db.Timestamp) then
		skipupdate = nil
		timestamp = tostring(temp[1].Timestamp)
		if string.len(timestamp) < 12 then timestamp = "0" .. timestamp end
		db.Timestamp = timestamp
		timestamp =  self:GetServerTime()
		self:out(LL["Updating DKP Points.."])
	end
	if not skipupdate then
		db.eqDKP = { }
		db.raidlist = { }
		if not db.raid or db.raid == L["NONE"] or #temp == 1 then db.raid = temp[1].Pool end
		for i, dkpray in pairs(temp) do
			if dkpray.Pool then
				db.raidlist[i] = dkpray.Pool
				db.eqDKP[dkpray.Pool] = { }
				db.eqDKP[dkpray.Pool]["eqDKPsite"] = dkpray.eqdkpsite
				db.eqDKP[dkpray.Pool]["prefix"] = dkpray.prefix
				if dkpray.Events then
					db.eqDKP[dkpray.Pool]["Events"] = { }
					for event, details in pairs(dkpray.Events) do
						db.eqDKP[dkpray.Pool]["Events"][event] = { }
						db.eqDKP[dkpray.Pool]["Events"][event] = { ["id"] = details.id, ["value"] = details.value}
					end
				end
				if dkpray.Aliases then
					db.eqDKP[dkpray.Pool]["Aliases"] = { }
					for alias, main in pairs(dkpray.Aliases) do
						db.eqDKP[dkpray.Pool]["Aliases"][alias] = main
						self:PlayerinDB(main)
						self:PlayerinDB(alias)
						local present = false
						if not db.info[main].aliases then db.info[main].aliases = { } end
						for numalias = 1 , #db.info[main].aliases do
							if db.info[main].aliases[numalias].alt == alias and db.info[main].aliases[numalias].raid == dkpray.Pool then
								present = true
							end
						end
						if not present then table.insert(db.info[main].aliases , {raid = dkpray.Pool, alt = alias}) end
					end
				end
			end
			if dkpray.Points then
				local count = 0
				for name, v in pairs(dkpray.Points) do
					if not db.info[name] then db.info[name] = { } end
					local earned, adjust, spent, balance = floor(v.earn * 100 + .5)/100, floor(v.adj * 100 + .5)/100, floor(v.spent * 100 + .5)/100, floor(v.bal * 100 + .5)/100
					db.info[name][dkpray.Pool] = {earned = earned + adjust, spent = spent, points = balance }
					if not db.info[name].class and v.class and v.class ~= "Unknown" then 
						local englishclass = BCR[v.class]
						if englishclass then
							db.info[name].class = upper(englishclass) 
						end
					end
					db.info[name].raidloot = 0
					db.info[name].lastupdate = timestamp
					count = count + 1
					if db.info[name].aliases then
						for m = 1, #db.info[name].aliases do
							if db.info[name].aliases[m].raid == db.raid then
								local alt = db.info[name].aliases[m].alt
								local mainpoints = db.info[name][dkpray.Pool].points
								if not db.info[alt] then db.info[alt] = { } end
								db.info[alt][dkpray.Pool] = { 	points = mainpoints,
												earned = db.info[name][dkpray.Pool].earned,
												spent = db.info[name][dkpray.Pool].spent }
								db.info[alt].raidloot = 0
								db.info[alt].lastupdate = timestamp
								self:debug(fmt(LL["Added alias: %s of %s with %s DKP."], alt, name, mainpoints)) 
							end
						end
					end
				end
				self:out(fmt(LL["Added %s players from %s"], count, dkpray.Pool))
			end
		end
		MorgDKP2_DKP1 = nil
		MorgDKP2_DKP2 = nil
		MorgDKP2_DKP3 = nil
		MorgDKP2_DKP4 = nil
		MorgDKP2_DKP5 = nil
	end
	self:ToggleFrameUpdate("TooltipLDB", true)
end

function mod:DefaultDKP()
	local db = mdkp.db.profile
	if db.dbupdated == 0 then return end
	local name = UnitName("player")
	MorgDKP2_DKP1 = {
		Timestamp = 010120000800,
		Pool = L["NONE"],
		eqdkpsite = "aqdkp",
		prefix = "eqdkp_",
		Points = {
			[name] = {earn = 1397.00,spent = 405.00,adj = 0.00,bal = 992}
		},
		Aliases = {
			["TestAlias"] = name
		},
		Events = {
			["Void Reaver"] = {id = 77,value = 3.00}
		},
	}
end

function mod:TimeStampDiff(time1, time2)
	local hours1, hours2, min1, min2, day1, day2, month1, month2, year1, year2, total1, total2, timediff

	if time1 and time2 then	
		time1 = tostring(time1)
		time2 = tostring(time2)
		if strlen(time1) < 12 then time1 = "0" .. time1 end
		if strlen(time2) < 12 then time2 = "0" .. time2 end
		local _, _,  day1, month1, year1, hours1, min1 = string.find(time1, "(%d%d)(%d%d)(%d%d%d%d)(%d%d)(%d%d)")
		local _, _,  day2, month2, year2, hours2, min2 = string.find(time2, "(%d%d)(%d%d)(%d%d%d%d)(%d%d)(%d%d)")
		total1 = ((((((month1 * 30) + day1) * 24) + hours1) * 60 ) + min1)
		total2 = ((((((month2 * 30) + day2) * 24) + hours2) * 60 ) + min2)
		if year1 > year2 then return true
		elseif total1 > total2 then return true
		else return false
		end
    	end
    	return false
end

function mod:CullItemDatabase()
	local cull = self.db.profile.culllvl
	local culled = 0
	for item, data in pairs(db.items) do
		local _, iLink, iQuality, itemLevel, _, _, _, _, _, _ = GetItemInfo(item)
		if itemLevel and itemLevel < cull then
			if iQuality ~= 4 or (iQuality == 4 and itemLevel ~= 80 and itemLevel ~= 1) then 
				self:debug("Removing "..iLink)
				db.items[item] = nil
				culled = culled + 1
			end
		end
	end
	if culled > 0 then
		self:out(fmt(LL["Removed %s items from the database."], culled))
	end
end

function mod:CullMemberDatabase()
	local cull = self.db.profile.memculllvl
	cull = cull * 604800
	local currtime = self:GetServerTime()
	local culled = 0
	for name, data in pairs(db.info) do
		if self:TimeDiff(data.lastupdate, currtime) > cull then
			self:debug("Removing "..name.." from database")
			db.info[name] = nil
			culled = culled + 1
		end
	end
	if culled > 0 then
		self:out(fmt(LL["Removed %s members from the database."], culled))
	end
end
